/*****************************************
Tarea 3 Estructura de Datos y Algoritmos
Profesor: Agustin Gonzalez

Autores:
Kenneth Stuart
Eduardo Gonzalez
*****************************************/



#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <ctype.h>
#include "encabezados.h"



extern unsigned int f_hash (char *, int);
extern P_NODO crear_nodo(char *contenido, int frecuencia);
extern P_NODO buscar_nodo (NODO *lista,char *contenido);
extern NODO *insertar_nodo (NODO *lista, NODO *nodo,int pos);
extern NODO * eliminar_nodo (NODO * lista, char * contenido);
extern NODO * eliminar_nodo_pos (NODO * lista, int pos);
extern NODO **duplicar_tabla(int);
extern NODO * recorrer_tabla(NODO ** tabla, int largo);
extern NODO * sort_lista(NODO *);
extern NODO * ultimo_nodo(NODO *);


int escribir_texto();
int leer_archivo(char *);
void agrega_palabra_tabla( NODO ** tabla, int tamano, char * palabra, int frecuencia);



int t_hash = T_HASH;	// tamao de la tabla hash.
NODO **tabla_hash;  // tabla hash
int contador;	//variable que cuenta las palabras ingresadas en la tabla hash.



int main (int argc, char *argv[] )
{

  FILE *archivo; //manejar archivos
  NODO *nodo; // nodo de la lista



  // validar que haya exactamente 1 o 2 parmetros en la lnea de comandos

  if ( argc == 1) {
    printf("\nIngrese el Texto. Finalice con 'EOF' (CTRL+D) \n> ");
    escribir_texto();
    tabla_hash = duplicar_tabla(T_HASH); //se crea por primera vez la tabla.
    leer_archivo("texto.temporal");

  }

  else {
    // validar que el 1 parmetro sea un archivo
    if ( (archivo=fopen(argv[1],"r"))==NULL) {  // argv[1] es el nombre del archivo de texto.
      printf("Archivo de texto inexistente\nUso: %s [Archivo de Texto]\n",argv[0]);
      exit(1);
    }
    fclose(archivo);
    tabla_hash = duplicar_tabla(T_HASH); //se crea por primera vez la tabla.
    leer_archivo(argv[1]);
  }
  nodo = recorrer_tabla(tabla_hash,t_hash); //genera una unica lista con todos los nodos
  free(tabla_hash); //elimina la tabla
  nodo = sort_lista(nodo); //ordena los nodos
  nodo = ultimo_nodo(nodo); //invierte el orden,  para que sea de mayor a menor

  printf("\nOrdenamiento de palabras, segun su frecuencia\n");
  printf("\n\t/-----------------------------------------------\\\n");
  printf("\t|   PALABRA  \t\t\t|   FRECUENCIA\t|\n");
  printf("\t|-----------------------------------------------|\n");

  while(nodo!=NULL) {
    printf("\t| %-25.25s\t|\t%5.d\t|\n",nodo->palabra,nodo->frecuencia);
    nodo=nodo->anterior;
  }
  printf("\t\\-----------------------------------------------/\n");
  if (argc == 1) unlink("texto.temporal"); // borrra el archivo temporal
  return 0;
}


/*  Funcion para procesar el texto ingresado por teclado, lo escribe a un archivo temporal */
int escribir_texto() {
  char aux;
  int i;
  char linea[T_LINEA];
  FILE *texto;
  texto = fopen("texto.temporal","w");
  // lee uun caracter de consola hasta que lea @.
  for(i=0;;i++) {
    aux = getc(stdin);
    //if (aux=='@') break;
    if (aux==EOF) break;
    linea[i] = aux;
    if (i==T_LINEA-2) {
      linea[i+1] = '\0';
      //guarda lo leido hasta el momento en un archivo temporal
      fprintf(texto,"%s",linea);
      i = -1;
    }
  }
  linea[i] = '\0';
  //guarda lo leido hasta el momento en un archivo temporal
  fprintf(texto,"%s\n",linea);

  fclose(texto);
}



/* leer el archivo */
int leer_archivo(char *nombre)
{
  FILE *archivo;
  char aux;
  char leida[MAX_PALABRA];
  int i, k;
  NODO * resultado;
  NODO ** tabla_doble;
  contador=0;
  archivo = fopen(nombre,"r");
  do {
    i = 0;
    aux = fgetc(archivo); //leer un caracter
    while( (aux!=EOF) ){ //mientras no sea el final del archivo
      if (isspace(aux)) break; // no cosidera espacios ni retorno de carro
      if (ispunct(aux)) break; // no considera puntuaciones, asteristcos y otros
      if (isdigit(aux)) break; // no considera numeros
      if (isupper(aux)) aux = tolower(aux); // deja la letra en minuscula
      leida[i] = aux;
      i++;
      aux=fgetc(archivo);
    }
    leida[i]='\0';
    if (leida[0]!='\0')
      {
	if (contador == t_hash) //factor de carga igual a 1
	  {
	    tabla_doble = duplicar_tabla(2 * t_hash); //crea una nueva tabla
	    for (k=0; k<t_hash;k++)
	      {
		while(tabla_hash[k]!=NULL) //trasladar los nodos a la nueva tabla
		  {
		    agrega_palabra_tabla(tabla_doble,t_hash*2,tabla_hash[k]->palabra,tabla_hash[k]->frecuencia);

		    tabla_hash[k] = eliminar_nodo_pos( tabla_hash[k], 0);
		  }
	      }
	    free(tabla_hash); //libera la memoria usada por la tabla
	    tabla_hash = tabla_doble; //direcciona tabla_hash a la tabla nueva
	    t_hash = 2 * t_hash; //redefine el tamao de la tabla al doble
	  }
	agrega_palabra_tabla(tabla_hash,t_hash,leida,1); //agrega una palabra en la tabla.
	contador++;
      }
  } while(aux!=EOF);
  return 1;
}


void agrega_palabra_tabla( NODO ** tabla, int tamano, char * palabra, int frecuencia) {
  NODO * nodo;
  int hash = f_hash (palabra, tamano);
  if (frecuencia > 1) nodo = NULL; // evita busqueda redundante
  else nodo = buscar_nodo( tabla[hash] , palabra );
  if (nodo == NULL) { //no encontrado
    //crear nodo
    nodo = crear_nodo(palabra,frecuencia);
    //insertar nodo
    tabla[hash] = insertar_nodo( tabla[hash] , nodo , 0 );
  }
  else {
    nodo->frecuencia = nodo->frecuencia + frecuencia;
  }
}



